# Json exception
mail_thunder_cant_reformat_json_error = "Can't reformat json is type right?"
mail_thunder_wrong_json_data_error = "Can't parser json"
# Content data exception
mail_thunder_content_set_compiler_error = "When set compiler using content make an error"
mail_thunder_content_file_error = "MailThunder content file error"
mail_thunder_content_login_failed = "can't login with mail thunder content"
# Argparse
mail_thunder_service_file_error = "service param -s got the wrong data"
mail_thunder_login_error = "need set --user and --password to login"
mail_thunder_argparse_get_wrong_function = "get unknown function"
# Add command
add_command_exception: str = "command value type should be as method or function"
# Executor
executor_list_error: str = "executor receive wrong data list is none or wrong type"
# Json file exception
cant_execute_action_error: str = "cant execute action"
cant_generate_json_report: str = "can't generate json report"
cant_find_json_error: str = "cant find json file"
cant_save_json_error: str = "cant save json file"
action_is_null_error: str = "json action is null"
